@@ -17,6 +17,7 @@
 
 use crate::reader::serializer::TapeSerializer;
 use arrow_schema::ArrowError;
+use memchr::memchr2;
 use serde::Serialize;
 use std::fmt::Write;
 
@@ -180,7 +181,7 @@ impl<'a> Tape<'a> {
             TapeElement::Null => out.push_str("null"),
             TapeElement::I64(high) => match self.get(idx + 1) {
                 TapeElement::I32(low) => {
-                    let val = (high as i64) << 32 | (low as u32) as i64;
+                    let val = ((high as i64) << 32) | (low as u32) as i64;
                     let _ = write!(out, "{val}");
                     return idx + 2;
                 }
@@ -191,7 +192,7 @@ impl<'a> Tape<'a> {
             }
             TapeElement::F64(high) => match self.get(idx + 1) {
                 TapeElement::F32(low) => {
-                    let val = f64::from_bits((high as u64) << 32 | low as u64);
+                    let val = f64::from_bits(((high as u64) << 32) | low as u64);
                     let _ = write!(out, "{val}");
                     return idx + 2;
                 }
@@ -394,7 +395,7 @@ impl TapeDecoder {
                 }
                 // Decoding a string
                 DecoderState::String => {
-                    let s = iter.advance_until(|b| matches!(b, b'\\' | b'"'));
+                    let s = iter.skip_chrs(b'\\', b'"');
                     self.bytes.extend_from_slice(s);
 
                     match next!(iter) {
@@ -491,7 +492,7 @@ impl TapeDecoder {
                 // Parse a unicode escape sequence
                 DecoderState::Unicode(high, low, idx) => loop {
                     match *idx {
-                        0..=3 => *high = *high << 4 | parse_hex(next!(iter))? as u16,
+                        0..=3 => *high = (*high << 4) | parse_hex(next!(iter))? as u16,
                         4 => {
                             if let Some(c) = char::from_u32(*high as u32) {
                                 write_char(c, &mut self.bytes);
@@ -508,7 +509,7 @@ impl TapeDecoder {
                             b'u' => {}
                             b => return Err(err(b, "parsing surrogate pair unicode")),
                         },
-                        6..=9 => *low = *low << 4 | parse_hex(next!(iter))? as u16,
+                        6..=9 => *low = (*low << 4) | parse_hex(next!(iter))? as u16,
                         _ => {
                             let c = char_from_surrogate_pair(*low, *high)?;
                             write_char(c, &mut self.bytes);
@@ -582,7 +583,7 @@ impl TapeDecoder {
             self.bytes.len()
         );
 
-        let strings = std::str::from_utf8(&self.bytes)
+        let strings = simdutf8::basic::from_utf8(&self.bytes)
             .map_err(|_| ArrowError::JsonError("Encountered non-UTF-8 data".to_string()))?;
 
         for offset in self.offsets.iter().copied() {
@@ -615,29 +616,33 @@ impl TapeDecoder {
 }
 
 /// A wrapper around a slice iterator that provides some helper functionality
-struct BufIter<'a>(std::slice::Iter<'a, u8>);
+struct BufIter<'a> {
+    buf: &'a [u8],
+    pos: usize,
+}
 
 impl<'a> BufIter<'a> {
     fn new(buf: &'a [u8]) -> Self {
-        Self(buf.iter())
+        Self { buf, pos: 0 }
     }
 
+    #[inline]
     fn as_slice(&self) -> &'a [u8] {
-        self.0.as_slice()
+        &self.buf[self.pos..]
     }
 
+    #[inline]
     fn is_empty(&self) -> bool {
-        self.0.len() == 0
+        self.pos >= self.buf.len()
     }
 
     fn peek(&self) -> Option<u8> {
-        self.0.as_slice().first().copied()
+        self.buf.get(self.pos).copied()
     }
 
+    #[inline]
     fn advance(&mut self, skip: usize) {
-        for _ in 0..skip {
-            self.0.next();
-        }
+        self.pos += skip;
     }
 
     fn advance_until<F: FnMut(u8) -> bool>(&mut self, f: F) -> &[u8] {
@@ -654,6 +659,20 @@ impl<'a> BufIter<'a> {
         }
     }
 
+    fn skip_chrs(&mut self, c1: u8, c2: u8) -> &[u8] {
+        let s = self.as_slice();
+        match memchr2(c1, c2, s) {
+            Some(p) => {
+                self.advance(p);
+                &s[..p]
+            }
+            None => {
+                self.advance(s.len());
+                s
+            }
+        }
+    }
+
     fn skip_whitespace(&mut self) {
         self.advance_until(|b| !json_whitespace(b));
     }
@@ -663,11 +682,14 @@ impl Iterator for BufIter<'_> {
     type Item = u8;
 
     fn next(&mut self) -> Option<Self::Item> {
-        self.0.next().copied()
+        let b = self.peek();
+        self.pos += 1;
+        b
     }
 
     fn size_hint(&self) -> (usize, Option<usize>) {
-        self.0.size_hint()
+        let s = self.buf.len().checked_sub(self.pos).unwrap_or_default();
+        (s, Some(s))
     }
 }
 
@@ -683,7 +705,7 @@ fn err(b: u8, ctx: &str) -> ArrowError {
 
 /// Creates a character from an UTF-16 surrogate pair
 fn char_from_surrogate_pair(low: u16, high: u16) -> Result<char, ArrowError> {
-    let n = (((high - 0xD800) as u32) << 10 | (low - 0xDC00) as u32) + 0x1_0000;
+    let n = (((high - 0xD800) as u32) << 10) | ((low - 0xDC00) as u32 + 0x1_0000);
     char::from_u32(n)
         .ok_or_else(|| ArrowError::JsonError(format!("Invalid UTF-16 surrogate pair {n}")))
 }
