@@ -18,9 +18,7 @@
 #[macro_use]
 extern crate criterion;
 
-use arrow::array::{
-    Array, Decimal128Array, Decimal128Builder, Decimal256Array, Decimal256Builder,
-};
+use arrow::array::{Array, Decimal128Array, Decimal128Builder, Decimal256Array, Decimal256Builder};
 use criterion::Criterion;
 use rand::Rng;
 
