@@ -109,7 +109,7 @@ The vote will be open for at least 72 hours.
 [1]: https://github.com/apache/arrow-rs/tree/${release_hash}
 [2]: ${url}
 [3]: https://github.com/apache/arrow-rs/blob/${release_hash}/CHANGELOG.md
-[4]: https://github.com/apache/arrow-rs/blob/master/dev/release/verify-release-candidate.sh
+[4]: https://github.com/apache/arrow-rs/blob/main/dev/release/verify-release-candidate.sh
 MAIL
 echo "---------------------------------------------------------"
 
