@@ -18,14 +18,12 @@
 //! Contains implementations of the reader traits FileReader, RowGroupReader and PageReader
 //! Also contains implementations of the ChunkReader for files (with buffering) and byte arrays (RAM)
 
-use std::collections::VecDeque;
-use std::iter;
-use std::{fs::File, io::Read, path::Path, sync::Arc};
-
 use crate::basic::{Encoding, Type};
 use crate::bloom_filter::Sbbf;
 use crate::column::page::{Page, PageMetadata, PageReader};
 use crate::compression::{create_codec, Codec};
+#[cfg(feature = "encryption")]
+use crate::encryption::decrypt::{read_and_decrypt, CryptoContext};
 use crate::errors::{ParquetError, Result};
 use crate::file::page_index::offset_index::OffsetIndexMetaData;
 use crate::file::{
@@ -40,6 +38,9 @@ use crate::record::Row;
 use crate::schema::types::Type as SchemaType;
 use crate::thrift::{TCompactSliceInputProtocol, TSerializable};
 use bytes::Bytes;
+use std::collections::VecDeque;
+use std::iter;
+use std::{fs::File, io::Read, path::Path, sync::Arc};
 use thrift::protocol::TCompactInputProtocol;
 
 impl TryFrom<File> for SerializedFileReader<File> {
@@ -340,11 +341,53 @@ impl<R: 'static + ChunkReader> RowGroupReader for SerializedRowGroupReader<'_, R
 /// Reads a [`PageHeader`] from the provided [`Read`]
 pub(crate) fn read_page_header<T: Read>(input: &mut T) -> Result<PageHeader> {
     let mut prot = TCompactInputProtocol::new(input);
-    let page_header = PageHeader::read_from_in_protocol(&mut prot)?;
-    Ok(page_header)
+    Ok(PageHeader::read_from_in_protocol(&mut prot)?)
 }
 
-/// Reads a [`PageHeader`] from the provided [`Read`] returning the number of bytes read
+#[cfg(feature = "encryption")]
+pub(crate) fn read_encrypted_page_header<T: Read>(
+    input: &mut T,
+    crypto_context: Arc<CryptoContext>,
+) -> Result<PageHeader> {
+    let data_decryptor = crypto_context.data_decryptor();
+    let aad = crypto_context.create_page_header_aad()?;
+
+    let buf = read_and_decrypt(data_decryptor, input, aad.as_ref())?;
+
+    let mut prot = TCompactSliceInputProtocol::new(buf.as_slice());
+    Ok(PageHeader::read_from_in_protocol(&mut prot)?)
+}
+
+/// Reads a [`PageHeader`] from the provided [`Read`] returning the number of bytes read.
+/// If the page header is encrypted [`CryptoContext`] must be provided.
+#[cfg(feature = "encryption")]
+fn read_encrypted_page_header_len<T: Read>(
+    input: &mut T,
+    crypto_context: Option<Arc<CryptoContext>>,
+) -> Result<(usize, PageHeader)> {
+    /// A wrapper around a [`std::io::Read`] that keeps track of the bytes read
+    struct TrackedRead<R> {
+        inner: R,
+        bytes_read: usize,
+    }
+
+    impl<R: Read> Read for TrackedRead<R> {
+        fn read(&mut self, buf: &mut [u8]) -> std::io::Result<usize> {
+            let v = self.inner.read(buf)?;
+            self.bytes_read += v;
+            Ok(v)
+        }
+    }
+
+    let mut tracked = TrackedRead {
+        inner: input,
+        bytes_read: 0,
+    };
+    let header = read_encrypted_page_header(&mut tracked, crypto_context.unwrap())?;
+    Ok((tracked.bytes_read, header))
+}
+
+/// Reads a [`PageHeader`] from the provided [`Read`] returning the number of bytes read.
 fn read_page_header_len<T: Read>(input: &mut T) -> Result<(usize, PageHeader)> {
     /// A wrapper around a [`std::io::Read`] that keeps track of the bytes read
     struct TrackedRead<R> {
@@ -489,6 +532,12 @@ enum SerializedPageReaderState {
 
         // If the next page header has already been "peeked", we will cache it and it`s length here
         next_page_header: Option<Box<PageHeader>>,
+
+        /// The index of the data page within this column chunk
+        page_ordinal: usize,
+
+        /// Whether the next page is expected to be a dictionary page
+        require_dictionary: bool,
     },
     Pages {
         /// Remaining page locations
@@ -512,6 +561,10 @@ pub struct SerializedPageReader<R: ChunkReader> {
     physical_type: Type,
 
     state: SerializedPageReaderState,
+
+    /// Crypto context carrying objects required for decryption
+    #[cfg(feature = "encryption")]
+    crypto_context: Option<Arc<CryptoContext>>,
 }
 
 impl<R: ChunkReader> SerializedPageReader<R> {
@@ -526,6 +579,16 @@ impl<R: ChunkReader> SerializedPageReader<R> {
         SerializedPageReader::new_with_properties(reader, meta, total_rows, page_locations, props)
     }
 
+    /// Adds cryptographical information to the reader.
+    #[cfg(feature = "encryption")]
+    pub(crate) fn with_crypto_context(
+        mut self,
+        crypto_context: Option<Arc<CryptoContext>>,
+    ) -> Self {
+        self.crypto_context = crypto_context;
+        self
+    }
+
     /// Creates a new serialized page with custom options.
     pub fn new_with_properties(
         reader: Arc<R>,
@@ -558,14 +621,17 @@ impl<R: ChunkReader> SerializedPageReader<R> {
                 offset: start as usize,
                 remaining_bytes: len as usize,
                 next_page_header: None,
+                page_ordinal: 0,
+                require_dictionary: meta.dictionary_page_offset().is_some(),
             },
         };
-
         Ok(Self {
             reader,
             decompressor,
             state,
             physical_type: meta.column_type(),
+            #[cfg(feature = "encryption")]
+            crypto_context: None,
         })
     }
 
@@ -581,6 +647,7 @@ impl<R: ChunkReader> SerializedPageReader<R> {
                 offset,
                 remaining_bytes,
                 next_page_header,
+                ..
             } => {
                 loop {
                     if *remaining_bytes == 0 {
@@ -664,6 +731,8 @@ impl<R: ChunkReader> PageReader for SerializedPageReader<R> {
                     offset,
                     remaining_bytes: remaining,
                     next_page_header,
+                    page_ordinal,
+                    require_dictionary,
                 } => {
                     if *remaining == 0 {
                         return Ok(None);
@@ -673,7 +742,21 @@ impl<R: ChunkReader> PageReader for SerializedPageReader<R> {
                     let header = if let Some(header) = next_page_header.take() {
                         *header
                     } else {
+                        #[cfg(feature = "encryption")]
+                        let (header_len, header) = if self.crypto_context.is_some() {
+                            let crypto_context = page_crypto_context(
+                                &self.crypto_context,
+                                *page_ordinal,
+                                *require_dictionary,
+                            )?;
+                            read_encrypted_page_header_len(&mut read, crypto_context)?
+                        } else {
+                            read_page_header_len(&mut read)?
+                        };
+
+                        #[cfg(not(feature = "encryption"))]
                         let (header_len, header) = read_page_header_len(&mut read)?;
+
                         verify_page_header_len(header_len, *remaining)?;
                         *offset += header_len;
                         *remaining -= header_len;
@@ -703,12 +786,33 @@ impl<R: ChunkReader> PageReader for SerializedPageReader<R> {
                         ));
                     }
 
-                    decode_page(
+                    #[cfg(feature = "encryption")]
+                    let crypto_context = page_crypto_context(
+                        &self.crypto_context,
+                        *page_ordinal,
+                        *require_dictionary,
+                    )?;
+                    #[cfg(feature = "encryption")]
+                    let buffer: Vec<u8> = if let Some(crypto_context) = crypto_context {
+                        let decryptor = crypto_context.data_decryptor();
+                        let aad = crypto_context.create_page_aad()?;
+                        decryptor.decrypt(buffer.as_ref(), &aad)?
+                    } else {
+                        buffer
+                    };
+
+                    let page = decode_page(
                         header,
                         Bytes::from(buffer),
                         self.physical_type,
                         self.decompressor.as_mut(),
-                    )?
+                    )?;
+                    if page.is_data_page() {
+                        *page_ordinal += 1;
+                    } else if page.is_dictionary_page() {
+                        *require_dictionary = false;
+                    }
+                    page
                 }
                 SerializedPageReaderState::Pages {
                     page_locations,
@@ -751,6 +855,7 @@ impl<R: ChunkReader> PageReader for SerializedPageReader<R> {
                 offset,
                 remaining_bytes,
                 next_page_header,
+                ..
             } => {
                 loop {
                     if *remaining_bytes == 0 {
@@ -816,6 +921,7 @@ impl<R: ChunkReader> PageReader for SerializedPageReader<R> {
                 offset,
                 remaining_bytes,
                 next_page_header,
+                ..
             } => {
                 if let Some(buffered_header) = next_page_header.take() {
                     verify_page_size(
@@ -857,6 +963,21 @@ impl<R: ChunkReader> PageReader for SerializedPageReader<R> {
     }
 }
 
+#[cfg(feature = "encryption")]
+fn page_crypto_context(
+    crypto_context: &Option<Arc<CryptoContext>>,
+    page_ordinal: usize,
+    dictionary_page: bool,
+) -> Result<Option<Arc<CryptoContext>>> {
+    Ok(crypto_context.as_ref().map(|c| {
+        Arc::new(if dictionary_page {
+            c.for_dictionary_page()
+        } else {
+            c.with_page_ordinal(page_ordinal)
+        })
+    }))
+}
+
 #[cfg(test)]
 mod tests {
     use std::collections::HashSet;
